######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# R program for the basic Beta-Binomial example
# end sensitivity prior/data
# Chapter 2

# Version running with BRUGS

######################################################################################
######################################################################################


rm(list=ls()) 	
library("BRugs") 	
library(MASS)	


#########################################################################
# Run basic Beta-Binomial model
#########################################################################


# Load/check model, load data and inits

model  <- "model_BetaBin_Basic.txt"
data   <- "data_BetaBin_Basic.txt"
inits1 <- "inits1_BetaBin_Basic.txt"
inits2 <- "inits2_BetaBin_Basic.txt"
inits3 <- "inits3_BetaBin_Basic.txt"

modelCheck(model) 					# Check the model
modelData(data) 					# Load the data
modelCompile(numChains=3) 			# Compile the model with 3 chains
modelInits(c(inits1, inits2, inits3)) 	# Load the three initialisations 


# MCMC iterations

Nb_i <- 5000 		# Number of iterations for Burn in
N <- 10000 		# Number of iterations to be considered for inference
thin <- 1

# Burn-in period

modelUpdate(Nb_i, thin=thin)

# Inferences
# After Burn in period let's work inference assuming ergodic regime is reached

# Set the quantities we are interested in

samplesSet(c("pi", "pi_prior", "y_new"))

modelUpdate(N, thin = thin) 		# N iterations are kept


# -------------------------------------------------------------------------
# Results
# -------------------------------------------------------------------------

# Have a look on MCMC chains

windows()
Results <- samplesHistory("*", mfrow = c(2, 1))

# Have a look on posterior statistics

samplesStats("*") 


# --------------------------------------------------------
# Draw marginal posterior density for theta vs prior
# --------------------------------------------------------

# windows()

res <- 6
name_figure <- "PriorPosterior.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

cise.text <- 1.5

par(mfrow = c(1,1), oma = c(1,1,1,1))

pi <- samplesSample("pi")
pi_prior <- samplesSample("pi_prior")

plot(x=0, y=-1, ylim = c(0,max(density(pi)$y)*1.2), xlim = c(0,1), ylab = "", xlab = "", bty="n")
lines(density(pi), xlim = c(0,1), col = "black", lty = 1, lwd = 2)
lines(density(pi_prior), xlim = c(0,1), col = "black", lty = 2, lwd = 2)

# mtext(side=3, text = expression(paste("Prior and posterior distribution of ",pi)), bty="n", line=1, cex=1.5, font = 1)
mtext(side=1, text = expression(pi), bty="n", line=3, cex=cise.text, font = 1)
mtext(side=2, text = "Density", bty="n", line=3, cex=cise.text, font = 1)


legend(	legend = c("Prior","Posterior"),
 		col = c("black","black"), 	
 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = 1, bty ="n") 

dev.off()


# -----------------------------------------------------
# Joint posterior distribution for (theta,y_new)
# -----------------------------------------------------

# windows()

res <- 6
name_figure <- "JointPosterior.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

cise.text <- 1.5
cise.points <- 2

def.par <- par(no.readonly = TRUE) 		# save default, for resetting...

x <- samplesSample("pi")
y <- samplesSample("y_new")

indice <- which(x<=quantile(x, prob = 0.995))
x <- x[indice]
y <- y[indice]

xrange <- range(x) ; nbreaks.x=100
yrange <- range(y) ; nbreaks.y=100
nb.points = 5000

xhist <- hist(x, breaks=seq(xrange[1],xrange[2],length.out=nbreaks.x), plot=FALSE)
yhist <- hist(y, breaks=seq(yrange[1],yrange[2],length.out=nbreaks.y), plot=FALSE)

nf <- layout(matrix(c(2,0,1,3),2,2,byrow=TRUE), c(3,1), c(1,3), TRUE)
layout.show(nf)

par(mar=c(5,5,1,1))
plot(x[1:nb.points], y[1:nb.points], xlim=xrange, ylim=yrange, xlab="", ylab="", pch=".", cex=cise.points)
# dens2d <- kde2d(x=x, y=y, n = 100)
# contour(dens2d , nlevels = 3, drawlabels=F, col = "blue", lw = 2, add = T) 
mtext(text=expression(pi), side=1, line=3, cex = cise.text)
mtext(text=expression(y^new), side=2, line=3, cex = cise.text)

par(mar=c(0,5,3,1))
barplot(xhist$density, axes=FALSE, space=0, horiz=FALSE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for ",pi)), side=3, line=1, cex = cise.text)

par(mar=c(5,0,1,3))
barplot(yhist$density, axes=FALSE, space=0, horiz=TRUE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for ",y^new)), side=4, line = 1, las = 0, cex = cise.text)

par(def.par)

dev.off()




#########################################################################
# Run sensitivity prior / data
#########################################################################

# Load/check model, load data and inits

model  <- "model_BetaBin_SensibPriorData.txt"
data   <- "data_BetaBin_SensibPriorData.txt"
inits1 <- "inits1_BetaBin_SensibPriorData.txt"
inits2 <- "inits2_BetaBin_SensibPriorData.txt"
inits3 <- "inits3_BetaBin_SensibPriorData.txt"

modelCheck(model) 					# Check the model
modelData(data) 					# Load the data
modelCompile(numChains=3) 			# Compile the model with 3 chains
modelInits(c(inits1, inits2, inits3)) 	# Load the three initialisations 


# MCMC iterations

Nb_i <- 5000 		# Number of iterations for Burn in
N <- 10000 		# Number of iterations to be considered for inference
thin <- 1

# Burn-in period

modelUpdate(Nb_i, thin = thin)

# Inferences
# Set the quantities we are interested in

samplesSet(c("pi", "pi_p"))

# MCMC sample for inference (size N=10000 for each of the three chain)

modelUpdate(N, thin = thin) 		# N iterations are kept


# --------------------------------------------------------
# Draw marginal posterior density for all theta vs prior
# --------------------------------------------------------

# windows()

res <- 6
name_figure <- "SensibPriorData.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

cise.text <- 1.5
cise.ticks <- 1.2

par(mfrow = c(3,3), oma = c(3,2,1,1), mar = c(2,2,2,2))

for (i in 1:3) {
for (j in 1:3) {

pi <- paste("pi[",i,",",j,"]", sep="")
pi_p <- paste("pi_p[",i,",",j,"]", sep="")

pi <- samplesSample(pi)
pi_p <- samplesSample(pi_p)

plot(x=0, y=-1, ylim = c(0,10), xlim = c(0,1), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
lines(density(pi), xlim = c(0,1), col = "black", lty = 1, lwd = 1)
lines(density(pi_p), xlim = c(0,1), col = "black", lty = 2, lwd = 2)

if (i==3) { 
axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
mtext(side=1, text = expression(pi), bty="n", line=3, cex=cise.text, font = 1)
}
else axis(side = 1, tick = T, labels = F, cex.axis=1, las=1)

axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=cise.ticks)

if (j==1){ mtext(side = 2, text = "Density", bty="n", line=2, cex=cise.text, font = 1) }

}}

dev.off()

