######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# R program for the basic Beta-Binomial example
# end sensitivity prior/data
# Chapter 2

######################################################################################
######################################################################################


rm(list=ls()) 	
library("R2WinBUGS") 	
library(MASS)	
bugs.directory="c:/Maths/WinBUGS14/"
bugs.workingdir="D:/Dossiers/Enseignement/StatEnv2013/Journees/Journee2/matin/programmes/WB"
setwd(bugs.workingdir)


#########################################################################
# Run basic Beta-Binomial model
#########################################################################
###############################################################
# Define model
model  <- "model_BetaBin_Basic.txt"
sink(model)
cat("
model
{
# Prior
pi ~ dbeta (1,1)
# pi_prior will keep track of the prior distribution
pi_prior  ~ dbeta (1,1)
# Likelihood
y ~ dbin(pi,n)
# Predictive
y_new ~ dbin(pi,n_new)
}
",fill=TRUE)
sink()
# Bundle data
win.data <- list(n = 167, y = 16, n_new = 31)

# Inits function
inits <- function(){
  list(pi = runif(1), pi_prior=runif(1), y_new=runif(1,1,30))
} #
# Set the quantities we are interested in
# Parameters to estimate
params <- c("pi", "pi_prior", "y_new")
# MCMC settings
ni <- 15000 # TOTAL Number of iterations including Burn in
nb <- 5000 #Number of iterations for Burn in
nt <- 1
nc <- 3
# Load/check model, load data and inits
# Start Gibbs sampler
out1 <- bugs(data = win.data, inits = inits, parameters.to.save = params, 
             model.file = model,
             n.thin = nt, n.chains = nc, n.burnin = nb, n.iter = ni, debug = TRUE,
             bugs.directory= bugs.directory, working.directory=bugs.workingdir)

# Inferences
# After Burn in period let's work inference assuming ergodic regime is reached
print(out1$summary,dig=2)

# -------------------------------------------------------------------------
# Results
# -------------------------------------------------------------------------

# Have a look on MCMC chains

###################################################
read.bugs(c("CODA1.txt", "CODA2.txt", "Coda3.txt"), start=9901, end=10000)->zz100
read.bugs(c("CODA1.txt", "CODA2.txt", "Coda3.txt"))->zz
varnames(zz)
#zzz<-zz[,1:4,drop=F] #toutes les chaines variables 1  4
#Z <- ZZZ[[1]]                    #Select first chain

zz<-zz[,2:4,drop=F] #toutes les chaines variables "pi", "pi_prior", "y_new"
zz100<-zz100[,2:4,drop=F] # variables "pi", "pi_prior", "y_new"

par(mfrow=c(3,1))
traceplot(zz100)
traceplot(zz)

par(mfrow=c(3,1))
acfplot(zz,lag.max=15)

par(mfrow=(c(3,3)))
autocorr.plot(zz,auto.layout=F)

densityplot(zz100)
densityplot(zz)

# Have a look on posterior statistics

# --------------------------------------------------------
# Draw marginal posterior density for theta vs prior
# --------------------------------------------------------

# windows()

res <- 6
name_figure <- "PriorPosterior.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

cise.text <- 1.5

par(mfrow = c(1,1), oma = c(1,1,1,1))

pi <- out1$sims.list$pi
pi_prior <- out1$sims.list$pi_prior

plot(x=0, y=-1, ylim = c(0,max(density(pi)$y)*1.2), xlim = c(0,1), ylab = "", xlab = "", bty="n")
lines(density(pi), xlim = c(0,1), col = "black", lty = 1, lwd = 2)
lines(density(pi_prior), xlim = c(0,1), col = "black", lty = 2, lwd = 2)

# mtext(side=3, text = expression(paste("Prior and posterior distribution of ",pi)), bty="n", line=1, cex=1.5, font = 1)
mtext(side=1, text = expression(pi), bty="n", line=3, cex=cise.text, font = 1)
mtext(side=2, text = "Density", bty="n", line=3, cex=cise.text, font = 1)


legend(	legend = c("Prior","Posterior"),
 		col = c("black","black"), 	
 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = 1, bty ="n") 

dev.off()


# -----------------------------------------------------
# Joint posterior distribution for (theta,y_new)
# -----------------------------------------------------

# windows()

res <- 6
name_figure <- "JointPosterior.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

cise.text <- 1.5
cise.points <- 2

def.par <- par(no.readonly = TRUE) 		# save default, for resetting...

x <- out1$sims.list$pi
y <- out1$sims.list$y_new

indice <- which(x<=quantile(x, prob = 0.995))
x <- x[indice]
y <- y[indice]

xrange <- range(x) ; nbreaks.x=100
yrange <- range(y) ; nbreaks.y=100
nb.points = 5000

xhist <- hist(x, breaks=seq(xrange[1],xrange[2],length.out=nbreaks.x), plot=FALSE)
yhist <- hist(y, breaks=seq(yrange[1],yrange[2],length.out=nbreaks.y), plot=FALSE)

nf <- layout(matrix(c(2,0,1,3),2,2,byrow=TRUE), c(3,1), c(1,3), TRUE)
layout.show(nf)

par(mar=c(5,5,1,1))
plot(x[1:nb.points], y[1:nb.points], xlim=xrange, ylim=yrange, xlab="", ylab="", pch=".", cex=cise.points)
# dens2d <- kde2d(x=x, y=y, n = 100)
# contour(dens2d , nlevels = 3, drawlabels=F, col = "blue", lw = 2, add = T) 
mtext(text=expression(pi), side=1, line=3, cex = cise.text)
mtext(text=expression(y^new), side=2, line=3, cex = cise.text)

par(mar=c(0,5,3,1))
barplot(xhist$density, axes=FALSE, space=0, horiz=FALSE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for ",pi)), side=3, line=1, cex = cise.text)

par(mar=c(5,0,1,3))
barplot(yhist$density, axes=FALSE, space=0, horiz=TRUE, col = "lightgrey")
mtext(text=expression(paste("Marginal pdf for ",y^new)), side=4, line = 1, las = 0, cex = cise.text)

par(def.par)

dev.off()




#########################################################################
# Run sensitivity prior / data
#########################################################################

# Load/check model, load data and inits
###############################################################
# Relative influence of prior & data
# Define model
# Beta-Binomial model
model  <- "model_BetaBin_SensibPriorData.txt"
sink(model)
cat("
model
{ 
# 3 different priors on pi
for (i in 1:3)
{

# Each of the 3 priors is updated by 3 different data set
# (Binomial likelihood)
for (j in 1:3)
{
pi[i,j] ~ dbeta(alpha[i],beta[i])
pi_p[i,j] ~ dbeta(alpha[i],beta[i])
y[i,j] ~ dbin(pi[i,j], n[i,j])
} # end loop on j (data set)

} # end loop on i (prior on pi)

} 
",fill=TRUE)
sink()
# Bundle data
win.data <- list(
  alpha = c(1,9,1),
  beta = c(1,1,9),
  n = structure(.Data = c(
    3,3,3,
    15,15,15,
    150,150,150),
                .Dim = c(3,3)), 
  y = structure(.Data = c(
    1,1,1,
    5,5,5,
    50,50,50),
                .Dim = c(3,3))
)

# Inits function
inits <- function(){
  list(pi = matrix(runif(9),3,3), pi_p=matrix(runif(9),3,3))
} #
# Set the quantities we are interested in
# Parameters to estimate
params <- c("pi", "pi_p")
# MCMC settings
ni <- 15000 # TOTAL Number of iterations including Burn in
nb <- 5000 #Number of iterations for Burn in
nt <- 1
nc <- 3
# Load/check model, load data and inits
# Start Gibbs sampler
out2 <- bugs(data = win.data, inits = inits, parameters.to.save = params, 
             model.file = model,
             n.thin = nt, n.chains = nc, n.burnin = nb, n.iter = ni, debug = TRUE,
             bugs.directory= bugs.directory, working.directory=bugs.workingdir)

# Inferences
# After Burn in period let's work inference assuming ergodic regime is reached
print(out2$summary,dig=2)


# --------------------------------------------------------
# Draw marginal posterior density for all theta vs prior
# --------------------------------------------------------

# windows()

res <- 6
name_figure <- "SensibPriorData.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

cise.text <- 1.5
cise.ticks <- 1.2
pmat=out2$sims.matrix
par(mfrow = c(3,3), oma = c(3,2,1,1), mar = c(2,2,2,2))

for (i in 1:3) {
for (j in 1:3) {

pi <- paste("pi[",i,",",j,"]", sep="")
pi_p <- paste("pi_p[",i,",",j,"]", sep="")

pi <- pmat[,pi]
pi_p <- pmat[,pi_p]


plot(x=0, y=-1, ylim = c(0,10), xlim = c(0,1), ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
lines(density(pi), xlim = c(0,1), col = "black", lty = 1, lwd = 1)
lines(density(pi_p), xlim = c(0,1), col = "black", lty = 2, lwd = 2)

if (i==3) { 
axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
mtext(side=1, text = expression(pi), bty="n", line=3, cex=cise.text, font = 1)
}
else axis(side = 1, tick = T, labels = F, cex.axis=1, las=1)

axis(side = 2, tick = T, lwd.ticks = 0, labels = F, cex.axis=cise.ticks)

if (j==1){ mtext(side = 2, text = "Density", bty="n", line=2, cex=cise.text, font = 1) }

}}

dev.off()

